<?php

class ProviderLog
{
    private static $_log = array();

    public static function start($key = '')
    {
        ProviderLog::$_log[] = array(
            'key' => '[PHP] ' . $key,
            'type' => 'start',
            'time' => microtime(true) * 1000
        );
    }

    public static function end($key = '')
    {
        ProviderLog::$_log[] = array(
            'key' => '[PHP] ' . $key,
            'type' => 'end',
            'time' => microtime(true) * 1000
        );
    }

    public static function getLog() {
        return ProviderLog::$_log;
    }
}

function getThemeVersions() {
    return array('prestashop' => _PS_VERSION_);
}

function setThemeHashes($current_theme_path, $data = array()) {
    ProviderLog::start('setThemeHashes');
    $hashesfile = $current_theme_path . '/export/hashes.json';
    $hashes_content = file_exists($hashesfile) ? FilesHelper::readFile($hashesfile) : '';
    $hashes = json_decode($hashes_content, true);

    foreach ($data as $file => $content) {
        if ('[DELETED]' === $content) {
            if (isset($hashes[$file]))
                unset($hashes[$file]);
        } else {
            $hashes[$file] = $content;
        }
    }

    FilesHelper::writeFile($hashesfile, json_encode($hashes));
    ProviderLog::end('setThemeHashes');
}

function getThemeHashes($current_theme_path) {
    ProviderLog::start('getThemeHashes');
    $hashesfile = $current_theme_path . '/export/hashes.json';
    $result = getJsonContent($hashesfile);
    ProviderLog::end('getThemeHashes');
    return $result;
}

function setThemeCache($current_theme_path, $data = array()) {
    ProviderLog::start('setThemeCache');
    $cachefile = $current_theme_path . '/export/cache.json';
    $cache_content = file_exists($cachefile) ? FilesHelper::readFile($cachefile) : '';
    $cache = json_decode($cache_content, true);

    if (is_array($data)) {
        foreach ($data as $control => $storage) {
            if (!is_array($storage))
                continue;

            foreach ($storage as $file => $content) {
                if ('[DELETED]' === $content) {
                    if (isset($cache[$control]) && isset($cache[$control][$file]))
                        unset($cache[$control][$file]);
                } else {
                    $cache[$control][$file] = $content;
                }
            }
        }
    }

    FilesHelper::writeFile($cachefile, json_encode($cache));
    ProviderLog::end('setThemeCache');
}

function getThemeCache($current_theme_path) {
    ProviderLog::start('getThemeCache');
    $cachefile = $current_theme_path . '/export/cache.json';
    $result = getJsonContent($cachefile);
    ProviderLog::end('getThemeCache');
    return $result;
}

function getSliderDefaultSettings($current_theme_path) {
    $path = $current_theme_path . '/modules/slider.json';
    return getJsonContent($path, true);
}

function getJsonContent($path, $assoc = false) {
    ProviderLog::start('getJsonContent');
    $result = json_decode((file_exists($path) ? FilesHelper::readFile($path) : '{}'), $assoc);
    ProviderLog::end('getJsonContent');
    return $result;
}

function getPreviewChangedFiles($current_theme_path) {
    ProviderLog::start('getPreviewChangedFiles');
    $changed_files_file = $current_theme_path . '/export/preview_changed_files.json';
    $result = getJsonContent($changed_files_file, true);
    ProviderLog::end('getPreviewChangedFiles');
    return $result;
}

function setPreviewChangedFiles($base_theme_path, $preview_theme_path, $changed_files) {
    ProviderLog::start('setPreviewChangedFiles');
    $changed_files_file = $base_theme_path . '/export/preview_changed_files.json';
    $_changed_files = $changed_files;
    $changed_files = array();
    foreach($_changed_files as $path) {
        $file = FilesHelper::normalizePath($path);
        if (substr($file, 0, strlen($preview_theme_path)) == $preview_theme_path)
            $changed_files[] = substr($file, strlen($preview_theme_path));
        else
            $changed_files[] = $file;
    }
    $changed_files_file_content = json_encode(array_unique($changed_files));
    FilesHelper::writeFile($changed_files_file, $changed_files_file_content);
    ProviderLog::end('setPreviewChangedFiles');
}

function saveProject($themeName, $projectData) {
    ProviderLog::start('saveProject');
    $projectPath = getProjectPath($themeName);
    $saveData = array(
        'projectData' => $projectData
    );
    FilesHelper::writeFile($projectPath, json_encode($saveData));
    ProviderLog::end('saveProject');
}

function getProject($themeName) {
    ProviderLog::start('getProject');
    $projectPath = getProjectPath($themeName);
    $result = getJsonContent($projectPath);
    ProviderLog::end('getProject');
    return $result;
}

function getThemeTemplates($baseUrl, $themeName) {
    $templates = array();

    $baseUrl = $baseUrl . '?theme_name=' . $themeName;
    $templates['home'] = $baseUrl;
    $templates['default'] = $baseUrl . '&controller=address';

    $categoryId = Designer::getFirstCategory();
    $categoryPath = $categoryId > 0 ? "&id_category=$categoryId&controller=category" : "&missingContent=category";
    $templates['products'] = $baseUrl . $categoryPath;

    $productId = Designer::getFirstProduct();
    $productPath = $productId > 0 ? "&id_product=$productId&controller=product" : "&missingContent=product";
    $templates['productOverview'] = $baseUrl . $productPath;

    $cmsId = Designer::getFirstCMSPage();
    $cmsPagePath = $cmsId > 0 ? "&id_cms=$cmsId&controller=cms&id_lang=1" : "&missingContent=cms";
    $templates['pageTemplate'] = $baseUrl . $cmsPagePath;

    $shoppingCart = '&controller=order';
    $templates['shoppingCartTemplate'] = $baseUrl . $shoppingCart;

    $notFound = '&controller=404';
    $templates['template404'] = $baseUrl . $notFound;

    return $templates;
}
/*
 * Gets Urls for PrestaShop CMS
 */
function getPSUrls($baseUrl, $themeName, $adminDir, $token) {
    $controller = version_compare(_PS_VERSION_,'1.6','<') ? 'AdminHome' : 'AdminDashboard';
    return array(
        'admin' => $baseUrl . $adminDir . '/index.php',
        'preview' => $baseUrl . '?theme_name=' . $themeName
    );
}

function theme_get_manifest_version($themeName, $manifest = false) {
    if (false === $manifest) {
        $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
        $manifest_path = $themeDir . '/export/manifest.version';
        if (!file_exists($manifest_path))
            return 0;
        $version = FilesHelper::readFile($manifest_path);
        if (false === $version)
            return 0;
        return trim($version);
    }

    if (false === $manifest)
        return 0;

    if (!preg_match('#\#ver:(\d+)#i', $manifest, $matches))
        return 0;

    return trim($matches[1]);
}

function theme_get_manifests_dir() {
    $base_upload_dir = FilesHelper::normalizePath(_PS_DOWNLOAD_DIR_);
    return $base_upload_dir . '/manifests';
}

function theme_save_manifest($themeName, $manifest) {
    $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
    $version = theme_get_manifest_version($themeName, $manifest);
    FilesHelper::writeFile($themeDir . '/export/' . $version, $manifest);
    FilesHelper::writeFile($themeDir . '/export/manifest.version', $version);
}

function theme_load_manifest($version) {
    $manifests_dir = theme_get_manifests_dir();
    if (file_exists("$manifests_dir/$version")) {
        $content = FilesHelper::readFile("$manifests_dir/$version");
        $protocol = Tools::getShopProtocol();
        if ($protocol === 'https://')
            $content = str_replace('http://', 'https://', $content);
        return $content;
    }
    return false;
}

function theme_copy_manifest($themeName, $version) {
    $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
    $manifests_dir = theme_get_manifests_dir();
    $src = "$themeDir/export/$version";
    $dest = "$manifests_dir/$version";
    if (!file_exists($dest) && file_exists($src))
        FilesHelper::copyFile($src, $dest);
}

function getSessionDomain($themeName = '') {
    $domain = Tools::getIsset('domain') ? '&domain=' . trim(Tools::getValue('domain')) : '';
    $noCache = Tools::getIsset('noCache') ? '&noCache=' . trim(Tools::getValue('noCache')) : '';
    return $domain . $noCache;
}

function getDesktopParams() {
    $startup = Tools::getIsset('startup') ? '&startup=' . Tools::getValue('startup', 'false') : '';
    $desktop = Tools::getIsset('desktop') ? '&desktop=' . Tools::getValue('desktop', 'false') : '';
    return $startup . $desktop;
}

function moveInnerPreview($themeName) {
    $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
    $inner_preview_dir = $themeDir . '/' . $themeName . _PREVIEW_SUFFIX_;
    if (file_exists($inner_preview_dir)) {
        $previewThemeName = $themeName . _PREVIEW_SUFFIX_;
        $previewThemeDir = FilesHelper::normalizePath(getThemeDir($previewThemeName));
        if (is_dir($previewThemeDir))
            FilesHelper::removeDir($previewThemeDir);
        rename($inner_preview_dir, $previewThemeDir);
    }
}

function upload($themeName, $callback) {
    $filename = isset($_REQUEST['filename']) ? $_REQUEST['filename'] : false;
    $is_last = isset($_REQUEST['last']) ? $_REQUEST['last'] : false;
    $content_range = $_SERVER['HTTP_CONTENT_RANGE'];

    $base_upload_dir = FilesHelper::normalizePath(_PS_DOWNLOAD_DIR_);

    if (empty($base_upload_dir)) {
        $result = array(
            'status' => 'error',
            'message' => 'Upload folder error: ' . $base_upload_dir
        );
    } elseif (!isset($_FILES['chunk']) || !file_exists($_FILES['chunk']['tmp_name'])) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty chunk data'
        );
    } elseif (!$content_range && !$is_last) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty Content-Range header'
        );
    } elseif (!$filename) {
        $result = array(
            'status' => 'error',
            'message' => 'Empty file name'
        );
    } else {
        $tmp_path = $base_upload_dir . '/' . $filename . '.tmp';
        $range_begin = 0;

        if ($content_range) {
            list($range, $total) = explode('/', str_replace('bytes ', '', $content_range));
            list($range_begin, $range_end) = explode('-', $range);
        }

        $file = fopen($tmp_path, 'c');

        if (flock($file, LOCK_EX)) {

            fseek($file, (int) $range_begin);
            fwrite($file, file_get_contents($_FILES['chunk']['tmp_name']));

            flock($file, LOCK_UN);
            fclose($file);
        }

        if ($is_last) {
            return call_user_func($callback, $themeName, $filename, $tmp_path);
        }

        $result = array('status' => 'processed');
    }

    return $result;
}

function uploadFile($themeName, $filename, $tmp_path) {
    $themeDir = FilesHelper::normalizePath(getThemeDir($themeName));
    $previewThemeDir = FilesHelper::normalizePath(getThemeDir($themeName . _PREVIEW_SUFFIX_));
    $base_dir = $previewThemeDir . '/img';
    
    $base_path = $base_dir . '/' . $filename;
    FilesHelper::deleteFile($base_path);
    FilesHelper::createDir($base_dir);
    FilesHelper::renameFile($tmp_path, $base_path);
    
    $changed_files = getPreviewChangedFiles($themeDir);
    $changed_files[] = $base_path;
    setPreviewChangedFiles($themeDir, $previewThemeDir, $changed_files);

    return array(
        'status' => 'done',
        'url' => Designer::getBaseUrlWithLastSlash() . 'themes/' . $themeName . '_preview/img/' . $filename
    );
}

function uploadTheme($themeName, $filename, $tmp_path) {
    $uniqid = uniqid();
    $sandbox = FilesHelper::normalizePath(_PS_CACHE_DIR_.'sandbox'.DIRECTORY_SEPARATOR.$uniqid) . '/';

    $uploadDir = $sandbox . ThemeInstallHelper::UPLOADED_THEME_DIR_NAME; // used in AdminThemesController
    $base_path = $uploadDir . '.zip';
    FilesHelper::createDir($sandbox);
    FilesHelper::renameFile($tmp_path, $base_path);

    $archive = new PclZip($base_path);
    deleteFromArchive($archive, 'modules', true); // We upload new theme, not modules to save module functionality

    $helper = new ThemeInstallHelper();
    if ($helper->install($base_path, $sandbox)) {
        $result = array(
            'status' => 'done',
            'path' => $base_path
        );
    } else {

        $themesDir = $uploadDir . '/themes';
        $files = FilesHelper::enumerate($themesDir, false); // not enumerateFiles method with directory checking 
        if (count($files) === 1 && is_dir($files[0]['path'])) {
            $themeName = str_replace("$themesDir/", '', $files[0]['path']);
        }

        if ($themeName && ($theme = Theme::getByDirectory($themeName)) && is_file($uploadDir . '/Config.xml')) {
            $themeDir =  $themesDir . '/' . $themeName;
            $previewThemeDir = $themeDir . '/' . $themeName . _PREVIEW_SUFFIX_;

            $newThemeName = checkThemeName($themeName);
            $newThemeDir = $themesDir . '/' . $newThemeName;

            $newThemePreviewDir = $themeDir . '/' . $newThemeName . _PREVIEW_SUFFIX_; // 1. rename inner preview folder
            processRename($themeDir, $previewThemeDir, $newThemeDir, $newThemePreviewDir, $newThemeName);
            $newThemePreviewDir = $newThemeDir . '/' . $newThemeName . _PREVIEW_SUFFIX_; // 2. now preview folder is correct

            FilesHelper::deleteFile($uploadDir . '/Config.xml');
            FilesHelper::copyFile($newThemePreviewDir .'/designer/Export/Config.xml', $uploadDir . '/Config.xml');

            unset($archive);
            FilesHelper::deleteFile($base_path);
            $archive = new PclZip($base_path);
            addToArchive($archive, $uploadDir, $uploadDir);
            $helper = new ThemeInstallHelper(); // new object with no errors
            if ($helper->install($base_path, $sandbox)) {
                $result = array(
                    'status' => 'done',
                    'path' => $base_path
                );
            } else {
                $result = array(
                    'status' => 'error',
                    'errors' => Tools::jsonEncode($helper->getErrors())
                );
            }
        } else {
            $result = array(
                'status' => 'error',
                'errors' => 'Incorrect archive file'
            );
        }
    }

    return $result;
}

function checkThemeName($themeName) {
    $toCheck = FilesHelper::normalizePath(_PS_ALL_THEMES_DIR_) . '/' . $themeName;

    while (file_exists($toCheck)) {
        preg_match('#(.*?)(\d*)$#', $themeName, $m);
        $themeName = $m[1];
        $suffix = (int) $m[2];
        $suffix++;
        $themeName .= $suffix;
        $toCheck = FilesHelper::normalizePath(_PS_ALL_THEMES_DIR_) . '/' . $themeName;
    };
    
    return $themeName;
}

function processRename($themeDir, $previewThemeDir, $newThemeDir, $newThemePreviewDir, $newThemeName, $rename = true) {
    save_config($previewThemeDir, $previewThemeDir, $newThemeName); // Check with upload archive & rename
    copyOrRename($previewThemeDir, $newThemePreviewDir, $rename);
    copyOrRename($themeDir, $newThemeDir, $rename);
}

function copyOrRename($src, $dest, $rename) {
    if (file_exists($src)) {
        if ($rename)
            FilesHelper::renameFile($src, $dest);
        else
            FilesHelper::copyRecursive($src, $dest);
    }
}

function getMaxRequestSize()
{
    $postSize = toBytes(ini_get('post_max_size'));
    $uploadSize = toBytes(ini_get('upload_max_filesize'));
    $memorySize = toBytes(ini_get('memory_limit'));

    return min($postSize, $uploadSize, $memorySize);
}

function toBytes($str) {
    $str = strtolower(trim($str));

    if ($str) {
        switch ($str[strlen($str) - 1]) {
            case 'g':
                $str *= 1024;
            case 'm':
                $str *= 1024;
            case 'k':
                $str *= 1024;
        }
    }

    return intval($str);
}

function get_memory_limit() {
    if (!function_exists('ini_get'))
        return -1;
    return toBytes(ini_get('memory_limit'));
}

function theme_out_of_memory_handler($cannot_allocate) {
    if ($cannot_allocate) {
        $msg = <<<EOL
            <h3>PHP Memory Configuration Error</h3>

            <p>Themler requires at least 64Mb of PHP memory. Please increase your PHP memory to continue.
            For more information, please check this <a href="http://answers.billiondigital.com/articles/5826/out-of-memory" target="_blank">link</a>.</p>
EOL;
    } else {
        $current_memory = get_memory_limit() / 1024 / 1024 . 'Mb';
        $msg = <<<EOL
            <h3>PHP Memory Configuration Error</h3>

            <p>Themler requires at least 64Mb of PHP memory (you have "$current_memory"). Please increase your PHP memory to continue.
            For more information, please check this <a href="http://answers.billiondigital.com/articles/5826/out-of-memory" target="_blank">link</a>.</p>
EOL;
    }
    $page = buildErrorsPage($msg);
    die($page);
}

//http://stackoverflow.com/questions/2726524/can-you-unregister-a-shutdown-function
function theme_memory_limit_shutdown() {
    $error = error_get_last();
    if ($error && $error['type'] === E_ERROR && !isset($GLOBALS['memory_test_passed'])) {
        theme_out_of_memory_handler(true);
    }
}

function theme_test_memory_size() {
    // try to allocate 16Mb
    $alloc_bytes = 16 * 1024 * 1024;
    register_shutdown_function('theme_memory_limit_shutdown');

    $tmp = @str_repeat('.', $alloc_bytes);
    unset($tmp);
    $GLOBALS['memory_test_passed'] = true;

    return true;
}

function theme_check_memory_limit($test_alloc_memory = false) {
    $need_memory_size = 64 * 1024 * 1024;
    $memory = get_memory_limit();

    // can't retrieve memory limit option
    if (-1 == $memory)
        return;

    // try to increase limit
    if ($memory < $need_memory_size) {
        if(!function_exists('ini_set'))
            theme_out_of_memory_handler(false);

        $ret = ini_set('memory_limit', '64M');
        if (!$ret)
            theme_out_of_memory_handler(false);
    }

    // check real limits
    if ($test_alloc_memory)
        theme_test_memory_size();
}

function buildErrorsPage($msg) {
    $output = <<<EOL
<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" lang="en-US">
    <head>
        <style>
            html {
                background: #eee;
            }
            body {
                background: #fff;
                color: #333;
                font-family: "Open Sans", sans-serif;
                margin: 2em auto;
                padding: 1em 2em;
                max-width: 700px;
                -webkit-box-shadow: 0 1px 3px rgba(0,0,0,0.13);
                box-shadow: 0 1px 3px rgba(0,0,0,0.13);
                word-break: break-word;
            }
            h3 {
                border-bottom: 1px solid #dadada;
                clear: both;
                color: #666;
                font: 24px "Open Sans", sans-serif;
                margin: 30px 0 0 0;
                padding: 0;
                padding-bottom: 7px;
            }
            #error-page {
                margin-top: 50px;
            }
            #error-page p {
                font-size: 14px;
                line-height: 1.5;
                margin: 25px 0 20px;
            }
            #error-page code {
                font-family: Consolas, Monaco, monospace;
            }
            ul li {
                margin-bottom: 10px;
                font-size: 14px ;
            }
            a {
                color: #21759B;
                text-decoration: none;
            }
            a:hover {
                color: #D54E21;
            }
        </style>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <title>PrestaShop</title>
    </head>
    <body id="error-page">
        $msg
    </body>
EOL;
    return $output;
}